package jp.ogakisoft.quiz.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import jp.ogakisoft.quiz.client.QuizCardPOJ;
import jp.ogakisoft.quiz.client.QuizIdPOJ;
import jp.ogakisoft.quiz.client.QuizService;
import au.com.bytecode.opencsv.CSVReader;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class QuizServiceImpl extends RemoteServiceServlet implements
		QuizService {
	public class Paths {
		private String path;

		public Paths(String path) {
			this.path = path;
		}

		public String getPath() {
			return this.path;
		}
	}

	private static final Logger log = Logger.getLogger(QuizServiceImpl.class
			.getName());
	private String[] bId;
	private String[] hId;
	private List<Paths> paths = new ArrayList<Paths>();
	private Properties prop;
	private int seq = 0;

	private QuizTitleJDO title = null;

	public QuizServiceImpl() {
		prop = new Properties();
		try {
			prop.load(new FileInputStream("WEB-INF/quizfile.properties"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		String str = prop.getProperty("quizfile.header");
		hId = str.split(",");
		str = prop.getProperty("quizfile.body");
		bId = str.split(",");
		try {
			File fi = new File(prop.getProperty("quizfile.path"));
			File[] fa = fi.listFiles();
			for (File f : fa) {
				String path = f.getAbsoluteFile().toString();
				paths.add(new Paths(path));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		str = prop.getProperty("quizfile.reload");
		if (str.toLowerCase().equals("yes")) {
			deleteAll();
			load();
		}
	}

	/*
	 * public String debug(String method) { if (method.equals("query")) { return
	 * get(""); } else if (method.equals("load")) { load(); return "done"; }
	 * else if (method.equals("delete")) { deleteAll(); return "done"; } return
	 * null; }
	 */
	@SuppressWarnings("unchecked")
	public void deleteAll() {
		PersistenceManager pm = PMF.get().getPersistenceManager();
		Query query = null;
		try {
			query = pm.newQuery("select from " + QuizJDO.class.getName());
			try {
				List<QuizJDO> result = (List<QuizJDO>) query.execute();
				for (QuizJDO q : result) {
					log.info("delete QuizJDO: id=" + q.getId());
					QuizJDO obj = pm.getObjectById(QuizJDO.class, q.getId());
					pm.deletePersistent(obj);
				}
			} finally {
				query.closeAll();
			}
			query = pm.newQuery("select from " + QuizTitleJDO.class.getName());
			try {
				List<QuizTitleJDO> titles = (List<QuizTitleJDO>) query
						.execute();
				for (QuizTitleJDO q : titles) {
					log.info("delete QuizTitleJDO: id=" + q.getId());
					QuizTitleJDO obj = pm.getObjectById(QuizTitleJDO.class, q
							.getId());
					pm.deletePersistent(obj);
				}
			} finally {
				query.closeAll();
			}
		} finally {
			pm.close();
		}
	}

	@SuppressWarnings("unchecked")
	public String get(String text) {
		log.info("get;" + text);
		StringBuffer sb = new StringBuffer();
		PersistenceManager pm = PMF.get().getPersistenceManager();
		Query query = null;
		try {
			String str = "select from " + QuizTitleJDO.class.getName();
			if (text != null)
				str = str + " " + text;
			log.info("query=" + str);
			query = pm.newQuery(str);
			List<QuizTitleJDO> list = (List<QuizTitleJDO>) query.execute();
			log.info("query result=" + list.size());
			for (QuizTitleJDO q : list) {
				sb.append(q.toString());
				log.info("get;title=" + q.toString());
			}
		} finally {
			query.closeAll();
		}
		try {
			String str = "select from " + QuizJDO.class.getName();
			if (str != null)
				str = str + " " + text;
			log.info("query=" + str);
			query = pm.newQuery(str);
			List<QuizJDO> list = (List<QuizJDO>) query.execute();
			log.info("query result=" + list.size());
			for (QuizJDO q : list) {
				sb.append(q.toString());
				log.info("get;card=" + q.toString());
			}
		} finally {
			query.closeAll();
		}
		return sb.toString();
	}

	@SuppressWarnings("unchecked")
	public List<QuizJDO> getEntries(String quizId, int page) {
		List<QuizJDO> results = new ArrayList<QuizJDO>(0);
		PersistenceManager pm = PMF.get().getPersistenceManager();
		String str = "select from " + QuizJDO.class.getName()
				+ " where quizId == '" + quizId + "'" + " && seq == '" + page
				+ "'";
		Query query = pm.newQuery(str);
		try {
			results = (List<QuizJDO>) query.execute();
		} finally {
			query.closeAll();
		}
		return results;
	}

	public List<Paths> getPaths() {
		return this.paths;
	}

	@SuppressWarnings("unchecked")
	private List<QuizTitleJDO> getTitles(String quizId) {
		List<QuizTitleJDO> results = new ArrayList<QuizTitleJDO>(0);
		PersistenceManager pm = PMF.get().getPersistenceManager();
		String str = "select from " + QuizTitleJDO.class.getName();
		if (quizId != null)
			str += " where quizId == '" + quizId + "'";
		Query query = pm.newQuery(str);
		query.setOrdering("quizId");
		try {
			results = (List<QuizTitleJDO>) query.execute();
		} finally {
			query.closeAll();
		}
		return results;
	}

	private String ifNotNull(String[] args, int index) {
		if (index < 0)
			return null;
		return args[index];
	}

	private int index(String[] args, String str, int start) {
		int pos = -1;
		for (int ix = start; ix < args.length; ix++) {
			if (args[ix].startsWith(str)) {
				pos = ix;
				break;
			}
		}
		return pos;
	}

	public void load() {
		List<Paths> paths = getPaths();
		for (Paths p : paths) {
			try {
				load(p.getPath());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void load(String path) throws Exception {
		log.info("load; path=" + path);
		// FileReader fr = new FileReader(path);
		InputStreamReader fr = new InputStreamReader(new FileInputStream(path),
				java.nio.charset.Charset.forName("UTF-8"));
		if (!fr.ready()) {
			log.warning("file not ready");
			return;
		}
		log.info("load; CSVReader; start");
		CSVReader reader = new CSVReader(fr);
		String[] nextLine, firstLine = null;
		int lines = 0;
		while ((nextLine = reader.readNext()) != null) {
			lines++;
			// log.info("load; CSVReader; readNext; lines=" + lines);
			// log.info("line=" + jp.ogakisoft.debug.Util.toString(nextLine));
			if (nextLine[0].startsWith("#")) {
				firstLine = nextLine.clone();
			} else if (firstLine != null) {
				setValue(firstLine, nextLine);
			}
			// if (maxLines > 0 && lines == (maxLines))
			// break;
		}
		log.info("load; CSVReader; end");
	}

	@Override
	public List<String> quizAllTitles() {
		List<String> titles = new ArrayList<String>(0);
		List<QuizTitleJDO> list = null;
		try {
			list = getTitles(null);
			for (QuizTitleJDO q : list) {
				titles.add(q.getQuizId() + ";" + q.getTitle());
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.severe(e.getMessage());
		}
		return titles;
	}

	@Override
	public QuizCardPOJ quizCard(String id, int page) {
		QuizCardPOJ card = new QuizCardPOJ();
		List<QuizJDO> list = null;
		try {
			list = getEntries(id, page);
			if (list != null && list.size() > 0) {
				QuizJDO quiz = (QuizJDO) list.get(0);
				card.setKind(quiz.getKind());
				card.setPoint(quiz.getPoint());
				card.setQuestion(quiz.getQuestion());
				card.setAnswer(quiz.getAnswer());
				card.setChoose(quiz.getChoose());
			} else {
				card = null;
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.severe(e.getMessage());
		}
		return card;
	}

	@Override
	public QuizIdPOJ quizTitle(String quizId) {
		QuizIdPOJ title = new QuizIdPOJ();
		List<QuizTitleJDO> list = null;
		try {
			list = getTitles(quizId);
			assert (list == null);
			if (list != null) {
				QuizTitleJDO jdo = (QuizTitleJDO) list.get(0);
				title.setQuizId(jdo.getQuizId());
				title.setVer(jdo.getVer());
				title.setTitle(jdo.getTitle());
				title.setMinPoint(jdo.getMinPoint());
				title.setMaxPoint(jdo.getMaxPoint());
				title.setTimeLimit(jdo.getTimeLimit());
				title.setCount(jdo.getCount());
				log.info(title.toString());
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.severe(e.getMessage());
		}
		return title;
	}

	private void setValue(String[] aId, String[] aVal) {
		log.info("setValue; start");
		PersistenceManager pm = PMF.get().getPersistenceManager();
		if (aId[0].startsWith("#")) {
			/*
			 * try { log.info("setValue; aId[0]=" + aId[0]);
			 * log.info("setValue; hId[0]=" + hId[0]); } catch (Exception e) {
			 * e.printStackTrace(); }
			 */
			if (aId[0].equals(hId[0])) {
				log.info("setValue; head");
				title = new QuizTitleJDO();
				title.setQuizId(ifNotNull(aVal, index(aId, hId[0], 0)));
				title.setVer(ifNotNull(aVal, index(aId, hId[1], 0)));
				title.setTitle(ifNotNull(aVal, index(aId, hId[2], 0)));
				title.setMinPoint(ifNotNull(aVal, index(aId, hId[3], 0)));
				title.setMaxPoint(ifNotNull(aVal, index(aId, hId[4], 0)));
				title.setTimeLimit(ifNotNull(aVal, index(aId, hId[5], 0)));
				title.setCount(ifNotNull(aVal, index(aId, hId[6], 0)));
				// title.setQuizId(aVal[0]);
				// title.setVer(aVal[1]);
				// title.setTitle(aVal[2]);
				// title.setMinPoint(aVal[3]);
				// title.setMaxPoint(aVal[4]);
				// title.setTimeLimit(aVal[5]);
				// title.setCount(aVal[6]);

				try {
					pm.makePersistent(title);
				} finally {
					pm.close();
				}
				seq = 0;
			} else if (aId[0].equals(bId[0])) {
				log.info("setValue; body");
				QuizJDO quiz = new QuizJDO();
				if (title != null)
					quiz.setQuizId(title.getQuizId());
				seq++;
				quiz.setSeq("" + (seq));
				quiz.setKind(ifNotNull(aVal, index(aId, bId[0], 0)));
				quiz.setPoint(ifNotNull(aVal, index(aId, bId[1], 0)));
				quiz.setQuestion(ifNotNull(aVal, index(aId, bId[2], 0)));
				quiz.setAnswer(ifNotNull(aVal, index(aId, bId[3], 0)));
				// quiz.setKind(aVal[0]);
				// quiz.setPoint(aVal[1]);
				// quiz.setQuestion(aVal[2]);
				// quiz.setAnswer(aVal[3]);
				int ix = index(aId, bId[4], 0);
				// int ix = 4;
				int count = 0;
				String[] choose = new String[aVal.length - ix];
				for (; ix < aVal.length; ix++) {
					if (!aVal[ix].trim().isEmpty()) {
						choose[count++] = aVal[ix];
					}
				}
				quiz.setChoose(choose);
				try {
					log.info("setValue; makePersistent");
					pm.makePersistent(quiz);
				} finally {
					pm.close();
				}
			}
		}
		log.info("setValue; end");
	}
}
